import 'package:lib/lib.dart';
import 'package:point/calendar_event/models/calendar_event_db_model.dart';
import 'package:point/calendar_event/models/calendar_event_response.dart';

class CalendarEventDatabase extends BaseDatabase<CalendarEventDbModel, CalendarEventResponse> {
  @override
  String get collectionId => "6619bc4edf8000989821";

  @override
  String get databaseId => "65fddef523b653ce0952";


  CalendarEventDatabase({required super.host, required super.projectId, required super.apiKey, required Future<User> Function(String) getUser}) :
    super(
      itemFromDbMap: (m) async => CalendarEventResponse.fromDbData(m, await getUser(m[CalendarEventDbModel.userIdDbFieldName])),
    );

  @override
  Future<DbResponse<CalendarEventResponse>> create(DbRequest<CalendarEventDbModel> request) {
    if(!request.requestingUser.teamIds.contains(request.item.teamId)) {
      throw UnauthorizedException();
    }
    
    return super.create(request);
  }

  @override
  Future<DbResponse<CalendarEventResponse>> get(DbRequest<String> request) async {
    var response = await super.get(request);
    if(!request.requestingUser.teamIds.contains(response.item.teamId)) {
      throw UnauthorizedException();
    }

    return response;
  }

  @override
  Stream<DbResponse<CalendarEventResponse>> getFromQueries(DbRequest<Iterable<String>> request) async* {
    await for(var queryResult in super.getFromQueries(request)) {
      if(request.requestingUser.teamIds.contains(queryResult.item.teamId)) {
        yield queryResult;
      }
    }
  }

  @override
  Stream<DbResponse<CalendarEventResponse>> getAll(DbRequest request) async* {
    await for(var response in super.getAll(request)) {
      if(request.requestingUser.teamIds.contains(response.item.teamId)) {
        yield response;
      }
    }
  }

  @override
  Future<DbResponse<CalendarEventResponse>> update(DbRequest<CalendarEventDbModel> request) async {
    var eventId = request.item.id;
    if (eventId == null) {
      throw RequiredArgumentMissing(RequestResponseVariables.idFieldName);
    }
    var response = await get(DbRequest(item: eventId, requestingUser: request.requestingUser));
    if (response.item.user.id != request.requestingUser.userId) {
      throw UnauthorizedException();
    }

    return super.update(request);
  }

  @override
  Future<DbResponse<CalendarEventResponse>> delete(DbRequest<String> request) async {
    var response = await get(request);
    if(response.item.user.id != request.requestingUser.userId) {
      throw UnauthorizedException();
    }

    return super.delete(request);
  }
}
