import 'package:lib/lib.dart';
import 'package:point/calendar_event/models/calendar_event.dart';

class CalendarEventResponse extends CalendarEvent implements ResponseModel {
  static const String userRequestResponseFieldName = 'user';

  final User _user;

  User get user => _user;

  CalendarEventResponse({required User user, super.id, required super.teamId, required super.title, super.description, required super.start, required super.end}) :
    _user = user;

  @override
  Map<String, dynamic> toResponseData() {
    return {
      RequestResponseVariables.idFieldName: id,
      userRequestResponseFieldName: user.toResponseData(),
      CalendarEvent.teamIdRequestResponseFieldName: teamId,
      CalendarEvent.titleRequestResponseFieldName: title,
      CalendarEvent.descriptionRequestResponseFieldName: description,
      CalendarEvent.startRequestResponseFieldName: start.toIso8601String(),
      CalendarEvent.endRequestResponseFieldName: end.toIso8601String(),
    };
  }

  factory CalendarEventResponse.fromDbData(Map<String, dynamic> map, User user) {
    return CalendarEventResponse(
      id: getOptionalMapItem(DatabaseVariables.idColumnName, map),
      user: user,
      teamId: getRequiredMapItem(CalendarEvent.teamIdDbFieldName, map),
      title: getRequiredMapItem(CalendarEvent. titleDbFieldName, map),
      description: getOptionalMapItem(CalendarEvent.descriptionDbFieldName, map),
      start: DateTime.parse(getRequiredMapItem(CalendarEvent.startDbFieldName, map)),
      end: DateTime.parse(getRequiredMapItem(CalendarEvent.endDbFieldName, map)),
    );
  }
}
