import 'dart:async';
import 'dart:io';
import 'package:point/calendar_event/calendar_event_api_endpoint.dart';
import 'package:point/membership/membership_api_endpoint.dart';
import 'package:point/membership/membership_service.dart';
import 'package:point/point/point_api_endpoint.dart';
import 'package:lib/lib.dart';
import 'package:point/route/route_api_endpoint.dart';
import 'package:point/schedule/schedule_api_endpoint.dart';

Future<dynamic> main(final context) async {
  final host = "https://${context.req.host}/v1";
  final apiKey = Platform.environment['FUNCTION_API_KEY'] ?? "";
  final projectId = AppwriteVariables.projectId;
  final String requestPath = context.req.path;

  ApiEndpoint apiEndpoint;
  switch (requestPath) {
    case ScheduleApiEndpoint.path:
      apiEndpoint = ScheduleApiEndpoint(trigger: AppwriteVariables.trigger(context));
      break;

    case PointApiEndpoint.path:
      var membershipService = getMembershipService(context, host, projectId, apiKey);
      var requestingUser = await getRequestingUser(context, membershipService);
      apiEndpoint = PointApiEndpoint(requestingUser: requestingUser, host: host, projectId: projectId, apiKey: apiKey);
      break;

    case RouteApiEndpoint.path:
      var membershipService = getMembershipService(context, host, projectId, apiKey);
      var requestingUser = await getRequestingUser(context, membershipService);
      apiEndpoint = RouteApiEndpoint(requestingUser: requestingUser, host: host, projectId: projectId, apiKey: apiKey);
      break;

    case MembershipApiEndpoint.path:
      var membershipService = getMembershipService(context, host, projectId, apiKey);
      apiEndpoint = MembershipApiEndpoint(membershipService: membershipService);

    case CalendarEventApiEndpoint.path:
      var membershipService = getMembershipService(context, host, projectId, apiKey);
      var requestingUser = await getRequestingUser(context, membershipService);
      var userHelper = UserHelper(host: host, projectId: projectId, apiKey: apiKey);
      apiEndpoint = CalendarEventApiEndpoint(requestingUser: requestingUser, host: host, projectId: projectId, apiKey: apiKey, getUser: userHelper.userFromId);

    default:
      throw HttpException("Path $requestPath not found", uri: Uri.parse(context.req.url));
  }

  var requestHandler = RequestHandler(apiEndpoint: apiEndpoint);
  return requestHandler.handleRequest(context);
}

MembershipService getMembershipService(final context, String host, String projectId, String apiKey) {
  final userId = AppwriteVariables.userId(context);
  return MembershipService(host: host, projectId: projectId, apiKey: apiKey, userId: userId);
}

Future<UserDetails> getRequestingUser(final context, MembershipService membershipService) async {
  final userId = AppwriteVariables.userId(context);
  return UserDetails(
    userId: userId,
    teamIds: (await membershipService.getMemberships().toList()).map((e) => e.teamId).toList(),
  );
}
