import 'package:lib/lib.dart';

class Point implements DbModel, ResponseModel {
  static const String latitudeDbFieldName = "latitude";
  static const String longitudeDbFieldName = "longitude";
  static const String latitudeRequestResponseFieldName = "latitude";
  static const String longitudeRequestResponseFieldName = "longitude";

  final String? _id;
  final num _latitude;
  final num _longitude;

  @override
  String? get id => _id;

  @override
  Permissions? get permissions => null;

  num get latitude => _latitude;
  num get longitude => _longitude;

  Point({String? id, required num latitude, required num longitude}) : 
    _id = id,
    _latitude = latitude,
    _longitude = longitude;
  
  @override
  Map<String, dynamic> toDbData() {
    return {
      latitudeDbFieldName: latitude,
      longitudeDbFieldName: longitude,
    };
  }
  
  @override
  Map<String, dynamic> toResponseData() {
    return {
      RequestResponseVariables.idFieldName: id,
      latitudeRequestResponseFieldName: latitude,
      longitudeRequestResponseFieldName: longitude,
    };
  }

  factory Point.fromRequest(Map<String, dynamic> map) {
    return Point(
      id: getOptionalMapItem(RequestResponseVariables.idFieldName, map),
      latitude: getRequiredMapItem(latitudeRequestResponseFieldName, map),
      longitude: getRequiredMapItem(longitudeRequestResponseFieldName, map),
    );
  }

  factory Point.fromDbData(Map<String, dynamic> map){
    return Point(
      id: getRequiredMapItem(DatabaseVariables.idColumnName, map),
      latitude: getRequiredMapItem(latitudeDbFieldName, map),
      longitude: getRequiredMapItem(longitudeDbFieldName, map),
    );
  }
}
