import 'package:dart_appwrite/dart_appwrite.dart';
import 'package:lib/lib.dart';
import 'package:point/point/point.dart';

class PointDatabase extends BaseDatabase<Point, Point> {
  @override
  String get databaseId => "65fddef523b653ce0952";

  @override
  String get collectionId => "65fddefc7f63cb43b15b";

  PointDatabase({required super.host, required super.projectId, required super.apiKey}) : 
    super(itemFromDbMap: (m) async => Point.fromDbData(m));

  @override
  Future<DbResponse<Point>> create(DbRequest<Point> request) async {
    var queries = List.of([
      Query.equal(Point.latitudeDbFieldName, request.item.latitude),
      Query.equal(Point.longitudeDbFieldName, request.item.longitude),
    ]);
    
    var equalItems = await getFromQueries(DbRequest(item: queries, requestingUser: request.requestingUser)).toList();
    if (equalItems.isNotEmpty) {
      return equalItems.first;
    }
    return super.create(request);
  }

  @override
  Future<DbResponse<Point>> update(DbRequest<Point> request) {
    throw DbActionForbiddenException("update", "point");
  }

  @override
  Future<DbResponse<Point>> delete(DbRequest<String> request) {
    throw DbActionForbiddenException("delete", "point"); // Maybe deletion is possible
  }

  @override
  Stream<DbResponse<Point>> getAll(DbRequest request) {
    throw DbActionForbiddenException("get all", "point");
  }
}
