import 'package:lib/lib.dart';
import 'package:point/point/point.dart';

class Route implements DbModel, ResponseModel {
  static const String userIdDbFieldName = "userId";
  static const String teamIdDbFieldName = "teamId";
  static const String pointsDbFieldName = "points";
  
  static const String userIdRequestResponseFieldName = "userId";
  static const String teamIdRequestResponseFieldName = "teamId";
  static const String pointsRequestResponseFieldName = "points";

  final String? _id;
  final List<Point> _points;
  final String _userId;
  final String? _teamId;

  @override
  String? get id => _id;

  @override
  Permissions? get permissions => null;

  Iterable<Point> get points => _points;
  String get userId => _userId;
  String? get teamId => _teamId;


  Route({String? id, required List<Point> points, required String userId, String? teamId}) :
    _id = id,
    _points = points,
    _userId = userId,
    _teamId = teamId;

  @override
  Map<String, dynamic> toDbData() {
    var pointIds = points.map((e) => e.id).toList(growable: false);
    return {
      pointsDbFieldName: pointIds,
      userIdDbFieldName : userId,
      teamIdDbFieldName : teamId,
    };
  }

  @override
  Map<String, dynamic> toResponseData() {
    return {
      RequestResponseVariables.idFieldName: id,
      pointsRequestResponseFieldName : points.map((e) => e.toResponseData()).toList(growable: false),
      userIdRequestResponseFieldName: userId,
      teamIdRequestResponseFieldName: teamId,
    };
  }

  factory Route.fromRequest(Map<String, dynamic> map, String userId) {
    return Route(
      id: getOptionalMapItem(RequestResponseVariables.idFieldName, map),
      points: getRequiredMapItem<Iterable>(pointsRequestResponseFieldName, map).map((e) => Point.fromRequest(e)).toList(growable: false),
      userId: userId,
      teamId: getOptionalMapItem(teamIdRequestResponseFieldName, map),
    );
  }

  factory Route.fromDbData(Map<String, dynamic> map) {
    return Route(
      id: getOptionalMapItem(DatabaseVariables.idColumnName, map),
      points: getRequiredMapItem<Iterable>(pointsDbFieldName, map).map((e) => Point.fromDbData(e)).toList(growable: false),
      userId: getRequiredMapItem(userIdDbFieldName, map),
      teamId: getOptionalMapItem(teamIdDbFieldName, map),
    );
  }
}
