import 'package:lib/lib.dart';
import 'package:point/point/point.dart';
import 'package:point/point/point_database.dart';
import 'package:point/route/route.dart';

class RouteDatabase extends BaseDatabase<Route, Route> {
  final PointDatabase _pointDatabase;

  @override
  String get collectionId => "6600a7f577154a949b4c";

  @override
  String get databaseId => "65fddef523b653ce0952";

  RouteDatabase({required super.host, required super.projectId, required super.apiKey}) :
    _pointDatabase = PointDatabase(host: host, projectId: projectId, apiKey: apiKey),
    super(itemFromDbMap: (m) async => Route.fromDbData(m));

  @override
  Future<DbResponse<Route>> create(DbRequest<Route> request) async {
    if (!request.requestingUser.teamIds.contains(request.item.teamId)){
      throw UnauthorizedException();
    }

    var points = <Point>[];
    for (var point in request.item.points) {
      // must be awaited instead of Future.wait to ensure order
      points.add((await _pointDatabase.create(DbRequest(item: point, requestingUser: request.requestingUser))).item);
    }

    request = DbRequest(
      item: Route(
        id: request.item.id,
        points: points,
        userId: request.item.userId,
        teamId: request.item.teamId,
      ),
      requestingUser: request.requestingUser,
    );
    return super.create(request);
  }

  @override
  Future<DbResponse<Route>> get(DbRequest<String> request) async {
    var response = await super.get(request);

    if(request.requestingUser.userId != response.item.userId) {
      throw UnauthorizedException();
    }

    return response;
  }

  @override
  Stream<DbResponse<Route>> getFromQueries(DbRequest<Iterable<String>> request) async* {
    await for(var queryResult in super.getFromQueries(request)) {
      if (request.requestingUser.userId == queryResult.item.userId) {
        yield queryResult;
      }
    }
  }

  @override
  Stream<DbResponse<Route>> getAll(DbRequest request) async* {
    await for(var response in super.getAll(request)) {
      if(request.requestingUser.userId == response.item.userId) {
        yield response;
      }
    }
  }

  @override
  Future<DbResponse<Route>> update(DbRequest<Route> request) async {
    var routeId = request.item.id;
    if (routeId == null) {
      throw RequiredArgumentMissing(RequestResponseVariables.idFieldName);
    }
    var routeResponse = await get(DbRequest(item: routeId, requestingUser: request.requestingUser));
    if (request.requestingUser.userId != routeResponse.item.userId) {
      throw UnauthorizedException();
    }
    return super.update(request);
  }

  @override
  Future<DbResponse<Route>> delete(DbRequest<String> request) async {
    var routeResponse = await get(request);
    if (request.requestingUser.userId != routeResponse.item.userId) {
      throw UnauthorizedException();
    }

    return super.delete(request);
  }
}
