import 'dart:convert';

import 'package:lib/src/base_database.dart';
import 'package:lib/src/interface/api_endpoint.dart';
import 'package:lib/src/interface/db_model.dart';
import 'package:lib/src/interface/response_model.dart';
import 'package:lib/src/request_response/db_request.dart';
import 'package:lib/src/user_details.dart';
import 'package:lib/src/utils/map_helper.dart';
import 'package:lib/src/utils/variables.dart';

class DbApiEndpoint<T extends DbModel, T1 extends ResponseModel> extends ApiEndpoint<T1> {
  final BaseDatabase<T, T1> database;
  final T Function(Map<String, dynamic>) itemGenerator;
  final UserDetails requestingUser;

  DbApiEndpoint({required this.database, required this.itemGenerator, required this.requestingUser});

  @override
  Future<Map<String, dynamic>> handleDelete(context) async {    
    var id = _idRequestFromContext(context);
    var response = await database.delete(id);
    return context.res.json(response.item.toResponseData());
  }

  @override
  Future<Map<String, dynamic>> handleGet(context) async {
    if(_hasBodyKey(context, RequestResponseVariables.idFieldName)) {
      var id = _idRequestFromContext(context);
      var response = await database.get(id);
      return context.res.json(response.item.toResponseData());
    }
    else if(_hasBodyKey(context, RequestResponseVariables.queriesFieldName)) {
      var queryRequest = _queryRequestFromContext(context);
      var response = await database.getFromQueries(queryRequest).toList();
      return listResponse(context, response.map((e) => e.item).toList());
    }
    else {
      var response = await database.getAll(DbRequest(item: null, requestingUser: requestingUser)).toList();
      return listResponse(context, response.map((e) => e.item).toList());
    }
  }

  @override
  Future<Map<String, dynamic>> handlePost(context) async {
    var item = _requestFromContext(context);
    var response = await database.create(item);
    return context.res.json(response.item.toResponseData());
  }

  @override
  Future<Map<String, dynamic>> handlePut(context) async {
    var item = _requestFromContext(context);
    var response = await database.update(item);
    return context.res.json(response.item.toResponseData());
  }

  DbRequest<T> _requestFromContext(final context) {
    return DbRequest(item: itemGenerator(jsonDecode(context.req.body)), requestingUser: requestingUser);
  }

  DbRequest<String> _idRequestFromContext(final context) {
    var map = jsonDecode(context.req.body);
    var id = getRequiredMapItem<String>(RequestResponseVariables.idFieldName, map);
    return DbRequest(item: id, requestingUser: requestingUser);
  }

  DbRequest<Iterable<String>> _queryRequestFromContext(final context) {
    var map = jsonDecode(context.req.body);
    var requestedQueries = getRequiredMapItem<Iterable<String>>(RequestResponseVariables.queriesFieldName, map);
    return DbRequest(item: requestedQueries, requestingUser: requestingUser);
  }

  bool _hasBodyKey(final context, String key) {
    var map = jsonDecode(context.req.body) as Map<String, dynamic>;
    return map.containsKey(key);
  }
}
