import 'package:dart_appwrite/dart_appwrite.dart';

class Permission {
  static final RegExp _permissionStringRegEx = RegExp(r'((?<action>\w+))\("((?<role>.+))"\)');
  final String role;
  final String action;

  String get dbPermissionString => "$action(\"$role\")";

  Permission({required this.role, required this.action});

  bool isUserPermitted(String userId, List<String>? userTeamIds) {
    // Not a complete check but is enough for this the current use case
    return role == Role.any() || role == Role.user(userId) || (userTeamIds != null && userTeamIds.any((teamId) => role == Role.team(teamId)));
  }

  factory Permission.parse(String permissionString) {
    var regexpMatch = _permissionStringRegEx.firstMatch(permissionString)!;    
    var action = regexpMatch.namedGroup("action")!;
    var role = regexpMatch.namedGroup("role")!;

    return Permission(
      action: action,
      role: role,
    );
  }
}